@@ -35,7 +35,6 @@ enum {
     DRIVER_POSTGRESQL,
     DRIVER_INFORMIX,
     DRIVER_TDS,
-    DRIVER_SYBASE,
     DRIVER_ORACLE,
     DRIVER_DB2,
 };
@@ -1745,13 +1744,13 @@ sql_fetchAbs(int cid, long rownum, ...)
 
 
 void
-getPrimaryKey(char *tname, int *part1, int *part2, int *part3)
+getPrimaryKey(char *tname, int *part1, int *part2, int *part3, int *part4)
 {
     int keyindex;
     SQLLEN pcbValue;
     char *dot;
 
-    *part1 = *part2 = *part3 = 0;
+    *part1 = *part2 = *part3 = *part4 = 0;
     newStatement();
     stmt_text = "get primary key";
     debugStatement();
@@ -1795,6 +1794,13 @@ getPrimaryKey(char *tname, int *part1, int *part2, int *part3)
 	goto abort;
     *part3 = keyindex;
 
+    rc = SQLFetch(hstmt);
+    if(rc == SQL_NO_DATA)
+	goto done;
+    if(rc)
+	goto abort;
+    *part4 = keyindex;
+
     goto done;
 
   abort:
@@ -1875,6 +1881,8 @@ fetchForeign(char *tname)
        NULL, SQL_NTS, NULL, SQL_NTS, NULL, SQL_NTS,
        NULL, SQL_NTS,
        (dot ? tname : NULL), SQL_NTS, (dot ? dot : tname), SQL_NTS);
+    if(dot)
+	dot[-1] = '.';
     if(rc)
 	goto abort;
 
