@@ -11,7 +11,7 @@
 bool
 sqlReadRows(const char *filename, char **bufptr)
 {
-    setError(316);
+    setError(MSG_DBNotCompiled);
     *bufptr = EMPTYSTRING;
     return false;
 }				/* sqlReadRows */
@@ -26,6 +26,16 @@ showColumns(void)
 {
 }				/* showColumns */
 
+void
+showForeign(void)
+{
+}				/* showForeign */
+
+bool
+showTables(void)
+{
+}				/* showTables */
+
 bool
 sqlDelRows(int start, int end)
 {
@@ -40,3 +50,17 @@ bool
 sqlAddRows(int ln)
 {
 }				/* sqlAddRows */
+
+bool
+ebConnect(void)
+{
+    setError(MSG_DBNotCompiled);
+    return false;
+}				/* ebConnect */
+
+int
+goSelect(int *startLine, char **rbuf)
+{
+    *rbuf = EMPTYSTRING;
+    return -1;
+}				/* goSelect */
