@@ -165,7 +165,7 @@ struct MACCOUNT {		/* pop3 account */
     char *login, *password, *from, *reply;
     char *inurl, *outurl;
     int inport, outport;
-    bool inssl, outssl;
+    uchar inssl, outssl;
 };
 
 struct MIMETYPE {
@@ -178,13 +178,15 @@ struct DBTABLE {
     char *name, *shortname;
     char *cols[MAXTCOLS];
     char ncols;
-    unsigned char key1, key2, key3;
+    unsigned char key1, key2, key3, key4;
     char *types;
+    char *nullable;
 };
 
 /* various globals */
 extern CURL *curl_handle;
 extern int debugLevel;		/* 0 to 9 */
+extern int displayLength;	/* when printing a line */
 extern int webTimeout, mailTimeout;
 extern int browseLine;		/* line number, for error reporting */
 extern bool ismc;		/* Is the program running as a mail client? */
@@ -202,6 +204,7 @@ extern bool errorExit;		/* exit on any error, for scripting purposes */
 extern bool isInteractive;
 extern volatile bool intFlag;	/* set this when interrupt signal is caught */
 extern bool binaryDetect;
+extern bool inputReadLine;
 extern bool listNA;		/* list nonascii chars */
 extern bool inInput;		/* reading line from standard in */
 extern int fileSize;		/* when reading/writing files */
@@ -210,7 +213,7 @@ extern int mssock;		/* mail server socket */
 extern int hcode;		/* http code, like 404 file not found */
 extern char herror[];		/* html error */
 extern char errorMsg[];		/* generated error message */
-extern char serverLine[MAXTTYLINE];	/* lines to and from the mail server */
+extern char serverLine[];	/* lines to and from the mail server */
 extern void *jcx;		/* javascript context */
 extern void *jwin;		/* javascript window object */
 extern void *jdoc;		/* javascript document object */
@@ -225,7 +228,6 @@ extern struct MIMETYPE mimetypes[];
 extern char *dbarea, *dblogin, *dbpw;	/* to log into the database */
 extern bool fetchBlobColumns;
 extern char *proxy_host;
-extern int proxy_port;
 extern bool caseInsensitive, searchStringsAll;
 extern bool undoable;		/* an undoable operation is taking place */
 extern bool allowRedirection;	/* from http code 301, or http refresh */
