@@ -271,15 +271,6 @@ writeAttachments(struct MHINFO *w)
     }
 }				/* writeAttachments */
 
-static void
-serverPutGet(const char *line, bool secure)
-{
-    if(!serverPutLine(line, secure))
-	showErrorAbort();
-    if(!serverGetLine(secure))
-	showErrorAbort();
-}				/* serverPutGet */
-
 void
 fetchMail(int account)
 {
@@ -304,27 +295,27 @@ fetchMail(int account)
 
     if(!mailConnect(host, a->inport, a->inssl))
 	showErrorAbort();
-    if(!serverGetLine(a->inssl))
+    if(!mailGetLine())
 	showErrorAbort();
     if(memcmp(serverLine, "+OK ", 4))
 	i_printfExit(MSG_BadPopIntro, serverLine);
     sprintf(serverLine, "user %s%s", login, eol);
-    serverPutGet(serverLine, a->inssl);
+    mailPutGetError(serverLine);
     if(pass) {			/* I think this is always required */
 	sprintf(serverLine, "pass %s%s", pass, eol);
-	serverPutGet(serverLine, a->inssl);
+	mailPutGetError(serverLine);
     }				/* password */
     if(memcmp(serverLine, "+OK", 3))
 	i_printfExit(MSG_PopNotComplete, serverLine);
 
 /* How many mail messages? */
-    serverPutGet("stat\r\n", a->inssl);
+    mailPutGetError("stat\r\n");
     if(memcmp(serverLine, "+OK ", 4))
 	i_printfExit(MSG_NoStatusMailBox, serverLine);
     nmsgs = atoi(serverLine + 4);
     if(!nmsgs) {
 	i_puts(MSG_NoMail);
-	serverClose(a->inssl);
+	mailClose();
 	exit(0);
     }
 
@@ -371,7 +362,7 @@ fetchMail(int account)
 	    cxSwitch(1, false);
 /* Now grab the entire message */
 	    sprintf(serverLine, "retr %d%s", m, eol);
-	    if(!serverPutLine(serverLine, a->inssl))
+	    if(!mailPutLine(serverLine, false))
 		showErrorAbort();
 	    retr1 = true;
 	    while(true) {
@@ -418,8 +409,12 @@ fetchMail(int account)
 	    }
 	    exact_l = j;
 
-/* get rid of the dos returns */
+/* get rid of the dos returns, and dot strip */
 	    for(j = k = 0; j < exact_l; ++j) {
+		if(!j && exact[j] == '.')
+		    continue;
+		if(j && exact[j] == '.' && exact[j - 1] == '\n')
+		    continue;
 		if(exact[j] == '\r' && j < exact_l - 1 && exact[j + 1] == '\n')
 		    continue;
 		exact[k++] = exact[j];
@@ -467,9 +462,9 @@ fetchMail(int account)
 		    nl();
 		} else if(!nattach &&	/* drop empty mail message */
 		   cw->dol -
-		   (lastMailInfo->subject != 0) -
-		   (lastMailInfo->from != 0) -
-		   (lastMailInfo->reply != 0) <= 1) {
+		   (strlen(lastMailInfo->subject) != 0) -
+		   (strlen(lastMailInfo->from) != 0) -
+		   (strlen(lastMailInfo->reply) != 0) <= 1) {
 		    redirect = "x";
 		    i_puts(MSG_Empty);
 		}
@@ -507,7 +502,7 @@ fetchMail(int account)
 			switch (key) {
 			case 'q':
 			    i_puts(MSG_Quit);
-			    serverClose(a->inssl);
+			    mailClose();
 			case 'x':
 			    exit(0);
 			case 'n':
@@ -628,9 +623,9 @@ fetchMail(int account)
  * So if you didn't mean to delete, type x to exit abruptly,
  * then fetch your mail again. */
 	    sprintf(serverLine, "dele %d%s", m, eol);
-	    if(!serverPutLine(serverLine, a->inssl))
+	    if(!mailPutLine(serverLine, false))
 		showErrorAbort();
-	    if(!serverGetLine(a->inssl))
+	    if(!mailGetLine())
 		i_printfExit(MSG_MailTimeOver);
 	    if(memcmp(serverLine, "+OK", 3))
 		i_printfExit(MSG_UnableDelMail, serverLine);
@@ -642,7 +637,7 @@ fetchMail(int account)
 
     if(zapMail)
 	printf("%d\n", nmsgs);
-    serverClose(a->inssl);
+    mailClose();
     exit(0);
 }				/* fetchMail */
 
@@ -1864,6 +1859,11 @@ setupReply(bool all)
 	    continue;
 	}
 
+	if(memEqualCI(p, "references:", 11)) {
+	    linetype[j] = 'v';
+	    continue;
+	}
+
 	if(memEqualCI(p, "reply to ", 9)) {
 	    linetype[j] = 'r';
 	    repln = j;
@@ -1887,19 +1887,36 @@ setupReply(bool all)
     }
 
 /* delete the lines we don't need */
-    for(j = 6; j >= 1; --j) {
-	if(!strchr("wfta", linetype[j]))
+    linetype[j] = 0;
+    for(--j; j >= 1; --j) {
+	if(strchr("srv", linetype[j]))
 	    continue;
 	delText(j, j);
 	strcpy(linetype + j, linetype + j + 1);
     }
 
 /* move reply to 1, if it isn't already there */
-    if(linetype[1] != 'r') {
+    repln = strchr(linetype, 'r') - linetype;
+    subln = strchr(linetype, 's') - linetype;
+    if(repln != 1) {
 	char *map = cw->map;
 	char swap[LNWIDTH];
 	char *q1 = map + LNWIDTH;
-	char *q2 = map + LNWIDTH * 2;
+	char *q2 = map + LNWIDTH * repln;
+	memcpy(swap, q1, LNWIDTH);
+	memcpy(q1, q2, LNWIDTH);
+	memcpy(q2, swap, LNWIDTH);
+	if(subln == 1)
+	    subln = repln;
+	repln = 1;
+    }
+
+    j = strlen(linetype) - 1;
+    if(j != subln) {
+	char *map = cw->map;
+	char swap[LNWIDTH];
+	char *q1 = map + LNWIDTH * j;
+	char *q2 = map + LNWIDTH * subln;
 	memcpy(swap, q1, LNWIDTH);
 	memcpy(q1, q2, LNWIDTH);
 	memcpy(q2, swap, LNWIDTH);
@@ -1945,7 +1962,7 @@ setupReply(bool all)
     if(t[1] == '>') {
 	i_puts(MSG_ReNoID);
     } else {
-	stringAndString(&out, &j, "references: <");
+	stringAndString(&out, &j, "References: <");
 	if(*s != '>') {
 	    stringAndBytes(&out, &j, s, t - s);
 	    stringAndString(&out, &j, "> <");
