@@ -1290,6 +1290,7 @@ andTranslate(const char *s, bool invisible)
 	"Utilde\0U",
 	"edot\0e",
 	"nbsp\0 ",
+	"shy\0-",
 	"frac13\01/3",
 	"frac23\02/3",
 	"plusmn\0+-",
@@ -1391,6 +1392,7 @@ andTranslate(const char *s, bool invisible)
 	"#8838\0subset of",
 	"#8839\0superset of",
 	"#9658\0*",
+	"#9837\0flat",
 	0
     };
 
@@ -1454,7 +1456,7 @@ andTranslate(const char *s, bool invisible)
 		    static char utfbuf[8];
 		    n = (uchar) c;
 		  n_utf8:
-		    uni2utf8(n, utfbuf);
+		    uni2utf8(n, (uchar *) utfbuf);
 		    r = utfbuf;
 		    goto putw;
 		}
