@@ -57,7 +57,6 @@ static int radioChecked_l;
 static char *preamble;
 static int preamble_l;
 
-static void htmlName(void);
 
 /* Switch from the linked list of tags to an array. */
 static void
@@ -254,6 +253,7 @@ struct htmlTag {
     bool bymail:1;		/* send by mail, rather than http */
     bool submitted:1;
     bool handler:1;
+    char subsup;		/* span turned into sup or sub */
     uchar itype;		/* input type = */
     short ninp;			/* number of nonhidden inputs */
     char *attrib;
@@ -430,7 +430,6 @@ htmlMeta(void)
     char **ptr;
     void *e;
 
-    htmlName();
     topTag->jv = e =
        domLink("Meta", topTag->name, topTag->id, 0, 0, "metas", jdoc, false);
     name = topTag->name;
@@ -526,6 +525,7 @@ formControl(bool namecheck)
     int itype = topTag->itype;
     int isradio = itype == INP_RADIO;
     int isselect = (itype == INP_SELECT) * 2;
+    char *myname = (topTag->name ? topTag->name : topTag->id);
 
     if(currentForm) {
 	topTag->controller = currentForm;
@@ -533,8 +533,7 @@ formControl(bool namecheck)
     } else if(itype != INP_BUTTON)
 	browseError(MSG_NotInForm2, topTag->info->desc);
 
-    htmlName();
-    if(namecheck && !topTag->name)
+    if(namecheck && !myname)
 	browseError(MSG_FieldNoName, topTag->info->desc);
 
     if(fo) {
@@ -574,7 +573,6 @@ static void
 htmlImage(void)
 {
     char *a;
-    htmlName();
     htmlHref("src");
     topTag->jv =
        domLink("Image", topTag->name, topTag->id, "src", topTag->href,
@@ -597,7 +595,6 @@ htmlForm(void)
     if(topTag->slash)
 	return;
     currentForm = topTag;
-    htmlName();
     htmlHref("action");
 
     a = htmlAttrVal(topAttrib, "method");
@@ -668,6 +665,7 @@ htmlInput(void)
 {
     int n = INP_TEXT;
     int len;
+    char *myname = (topTag->name ? topTag->name : topTag->id);
     char *s = htmlAttrVal(topAttrib, "type");
     if(s && *s) {
 	caseShift(s, 'l');
@@ -699,8 +697,7 @@ htmlInput(void)
     topTag->value = s;
     if(n >= INP_RADIO && htmlAttrPresent(topAttrib, "checked")) {
 	char namebuf[200];
-	if(n == INP_RADIO && topTag->name &&
-	   strlen(topTag->name) < sizeof (namebuf) - 3) {
+	if(n == INP_RADIO && myname && strlen(myname) < sizeof (namebuf) - 3) {
 	    if(!radioChecked) {
 		radioChecked = initString(&radioChecked_l);
 		stringAndChar(&radioChecked, &radioChecked_l, '|');
@@ -1075,6 +1072,24 @@ onloadGo(void *obj, const char *jsrc, const char *tagname)
     toPreamble(t->seqno, buf, jsrc, 0);
 }				/* onloadGo */
 
+/* Given a tag with an id attribute whose value is foo, generate a label
+ * <a name="foo">.  If the tag has no id attribute, this function is a no-op.
+*/
+
+static void
+htmlLabelID(char **browseText, int *browseTextLength)
+{
+    char buf[16];
+    char *id = htmlAttrVal(topAttrib, "id");
+    if(id) {
+	struct htmlTag *t = newTag("a");
+	int tagnum = t->seqno;
+	t->name = id;
+	sprintf(buf, "%c%d*", InternalCodeChar, tagnum);
+	stringAndString(browseText, browseTextLength, buf);
+    }
+}				/* htmlLabelID */
+
 /* Always returns a string, even if errors were found.
  * Internet web pages often contain errors!
  * This routine mucks with the passed-in string, and frees it
@@ -1274,6 +1289,10 @@ encodeTags(char *html)
 	if(slash && ti->bits & TAG_NOSLASH)
 	    continue;		/* negated version means nothing */
 
+/* just about any tag can have a name or id */
+	if(!slash)
+	    htmlName();
+
 /* Does this tag force the closure of an anchor? */
 	if(currentA && (action != TAGACT_A || !slash)) {
 	    if(open && open->clickable)
@@ -1364,6 +1383,9 @@ encodeTags(char *html)
 	    currentTitle = currentOpt = 0;
 	}
 
+/* Generate an html label for this tag if necessary. */
+	htmlLabelID(&new, &l);
+
 	switch (action) {
 	case TAGACT_INPUT:
 	    htmlInput();
@@ -1414,7 +1436,6 @@ encodeTags(char *html)
 		    a_href = true;
 		currentA = 0;
 	    } else {
-		htmlName();
 		htmlHref("href");
 		topTag->jv =
 		   domLink("Anchor", topTag->name, topTag->id, "href",
@@ -1451,14 +1472,12 @@ encodeTags(char *html)
 	    continue;
 
 	case TAGACT_HEAD:
-	    htmlName();
 	    topTag->jv =
 	       domLink("Head", topTag->name, topTag->id, 0, 0,
 	       "heads", jdoc, false);
 	    goto plainWithElements;
 
 	case TAGACT_BODY:
-	    htmlName();
 	    topTag->jv =
 	       domLink("Body", topTag->name, topTag->id, 0, 0,
 	       "bodies", jdoc, false);
@@ -1525,7 +1544,6 @@ encodeTags(char *html)
 
 	case TAGACT_TABLE:
 	    if(!slash) {
-		htmlName();
 		topTag->jv = to =
 		   domLink("Table", topTag->name, topTag->id, 0, 0,
 		   "tables", jdoc, false);
@@ -1551,7 +1569,6 @@ encodeTags(char *html)
 		++inrow;
 	    tdfirst = true;
 	    if((!slash) && (open = findOpenTag("table")) && open->jv) {
-		htmlName();
 		topTag->jv = to =
 		   domLink("Trow", topTag->name, topTag->id, 0, 0,
 		   "rows", open->jv, false);
@@ -1576,7 +1593,6 @@ encodeTags(char *html)
 		stringAndChar(&new, &l, '|');
 	    }
 	    if((open = findOpenTag("tr")) && open->jv) {
-		htmlName();
 		topTag->jv = to =
 		   domLink("Cell", topTag->name, topTag->id, 0, 0,
 		   "cells", open->jv, false);
@@ -1586,7 +1602,6 @@ encodeTags(char *html)
 
 	case TAGACT_DIV:
 	    if(!slash) {
-		htmlName();
 		topTag->jv =
 		   domLink("Div", topTag->name, topTag->id, 0, 0,
 		   "divs", jdoc, false);
@@ -1596,13 +1611,25 @@ encodeTags(char *html)
 
 	case TAGACT_SPAN:
 	    if(!slash) {
-		htmlName();
 		topTag->jv =
 		   domLink("Span", topTag->name, topTag->id, 0, 0,
 		   "spans", jdoc, false);
 		get_js_events();
-	    }
-	    goto nop;
+		a = htmlAttrVal(topAttrib, "class");
+		if(!a)
+		    goto nop;
+		caseShift(a, 'l');
+		if(stringEqual(a, "sup"))
+		    action = TAGACT_SUP;
+		if(stringEqual(a, "sub"))
+		    action = TAGACT_SUB;
+		nzFree(a);
+	    } else if(open && open->subsup)
+		action = open->subsup;
+	    if(action == TAGACT_SPAN)
+		goto nop;
+	    t->subsup = action;
+	    goto subsup;
 
 	case TAGACT_BR:
 	    if(lastact == TAGACT_TD)
@@ -1711,6 +1738,7 @@ encodeTags(char *html)
 
 	case TAGACT_SUP:
 	case TAGACT_SUB:
+	  subsup:
 	    if(!retainTag)
 		continue;
 	    t->retain = true;
@@ -1748,7 +1776,6 @@ encodeTags(char *html)
 
 	case TAGACT_AREA:
 	case TAGACT_FRAME:
-	    htmlName();
 	    if(action == TAGACT_FRAME) {
 		htmlHref("src");
 		topTag->jv =
@@ -1888,6 +1915,8 @@ encodeTags(char *html)
 			}
 			js_line = 1;
 			js_file = t->href;
+			nzFree(changeFileName);
+			changeFileName = NULL;
 		    }
 		}		/* fetch from the net */
 		if(javatext) {
@@ -3093,8 +3122,8 @@ infPush(int tagno, char **post_string)
 	}
     }
 
-/* if no action, the default is the current location */
-    if(!action) {
+/* if no action, or action is "#", the default is the current location */
+    if(!action || stringEqual(action, "#")) {
 	action = getBaseHref(form->seqno);
     }
 
