@@ -18,6 +18,7 @@
 CURL *curl_handle = NULL;
 char *serverData;
 int serverDataLen;
+static char errorText[CURL_ERROR_SIZE + 1];
 
 static void curl_setError(CURLcode curlret, const char *url);
 static bool ftpConnect(const char *url);
@@ -352,6 +353,8 @@ static char hexdigits[] = "0123456789abcdef";
  * All characters in the area between start and end, not including end,
  * are copied or transformed.
 
+ * Get rid of :/   curl can't handle it.
+
  * This function is used to sanitize user-supplied URLs.  */
 
 char *
@@ -361,11 +364,14 @@ copy_and_sanitize(const char *start, const char *end)
     char *new_copy = NULL;
     const char *in_pointer = NULL;
     char *out_pointer = NULL;
+    const char *portloc = NULL;
+
     for(in_pointer = start; in_pointer < end; in_pointer++)
 	if(*in_pointer <= 32)
 	    bytes_to_alloc += (ESCAPED_CHAR_LENGTH - 1);
     new_copy = allocMem(bytes_to_alloc);
     if(new_copy) {
+	char *frag, *params;
 	out_pointer = new_copy;
 	for(in_pointer = start; in_pointer < end; in_pointer++) {
 	    if(*in_pointer == '\\')
@@ -378,7 +384,23 @@ copy_and_sanitize(const char *start, const char *end)
 		*out_pointer++ = *in_pointer;
 	}
 	*out_pointer = '\0';
+/* excise #hash, required by some web servers */
+	frag = strchr(new_copy, '#');
+	if(frag) {
+	    params = strchr(new_copy, '?');
+	    if(params && params > frag)
+		strcpy(frag, params);
+	    else
+		*frag = 0;
+	}
+
+	getPortLocURL(new_copy, &portloc, 0);
+	if(portloc && !isdigit(portloc[1])) {
+	    const char *s = portloc + strcspn(portloc, "/?#\1");
+	    strcpy((char *)portloc, s);
+	}
     }
+
     return new_copy;
 }				/* copy_and_sanitize */
 
@@ -391,32 +413,37 @@ httpConnect(const char *from, const char *url)
     char *referrer = NULL;
     CURLcode curlret = CURLE_OK;
     struct curl_slist *custom_headers = NULL;
+    struct curl_slist *tmp_headers = NULL;
     char user[MAXUSERPASS], pass[MAXUSERPASS];
     char creds_buf[MAXUSERPASS * 2 + 1];	/* creds abr. for credentials */
     int creds_len = 0;
-    char *creds_ptr = NULL;
     bool still_fetching = true;
-    char *temp_url = NULL;
+    int ssl_version;
     const char *host;
     struct MIMETYPE *mt;
     const char *prot;
     char *cmd;
     char suffix[12];
     const char *post, *s;
-    char *postb;
+    char *postb = NULL;
     char *urlcopy = NULL;
-    int postb_l;
+    int postb_l = 0;
     bool transfer_status = false;
     int redirect_count = 0;
     bool name_changed = false;
 
     serverData = NULL;
     serverDataLen = 0;
-    creds_buf[0] = '\0';
+    strcpy(creds_buf, ":");	/* Flush stale username and password. */
 
     prot = getProtURL(url);
 
 /* See if the protocol is a recognized stream */
+    if(!prot) {
+	setError(MSG_WebProtBad, "(?)");
+	return false;
+    }
+
     if(stringEqualCI(prot, "http") || stringEqualCI(prot, "https")) {
 	;			/* ok for now */
     } else if(stringEqualCI(prot, "ftp")) {
@@ -465,7 +492,12 @@ httpConnect(const char *from, const char *url)
     }
 
 /* "Expect:" header causes some servers to lose.  Disable it. */
-    custom_headers = curl_slist_append(custom_headers, "Expect:");
+    tmp_headers = curl_slist_append(custom_headers, "Expect:");
+    /* Check for failure to allocate. */
+    if(tmp_headers == NULL)
+	i_printfExit(MSG_NoMem);
+    custom_headers = tmp_headers;
+
     post = strchr(url, '\1');
     postb = 0;
     if(post) {
@@ -485,8 +517,13 @@ httpConnect(const char *from, const char *url)
 	    s = strchr(post, '\r');
 	    stringAndBytes(&multipart_header, &multipart_header_len, post,
 	       s - post);
-	    custom_headers =
-	       curl_slist_append(custom_headers, multipart_header);
+	    tmp_headers = curl_slist_append(custom_headers, multipart_header);
+	    if(tmp_headers == NULL)
+		i_printfExit(MSG_NoMem);
+	    custom_headers = tmp_headers;
+	    /* curl_slist_append made a copy of multipart_header. */
+	    nzFree(multipart_header);
+
 	    memcpy(thisbound, post, s - post);
 	    thisbound[s - post] = 0;
 	    post = s + 2;
@@ -495,16 +532,16 @@ httpConnect(const char *from, const char *url)
 	curlret = curl_easy_setopt(curl_handle, CURLOPT_POSTFIELDS,
 	   (postb_l ? postb : post));
 	if(curlret != CURLE_OK)
-	    goto curl_fail;;
+	    goto curl_fail;
 	curlret = curl_easy_setopt(curl_handle, CURLOPT_POSTFIELDSIZE,
 	   postb_l ? postb_l : strlen(post));
 	if(curlret != CURLE_OK)
-	    goto curl_fail;;
+	    goto curl_fail;
     } else {
 	urlcopy = copy_and_sanitize(url, url + strlen(url));
 	curlret = curl_easy_setopt(curl_handle, CURLOPT_HTTPGET, 1);
 	if(curlret != CURLE_OK)
-	    goto curl_fail;;
+	    goto curl_fail;
     }
 
 
@@ -519,6 +556,7 @@ httpConnect(const char *from, const char *url)
 		post2 = currentReferrer + strlen(currentReferrer);
 	    if(post2 - currentReferrer >= 7 && !memcmp(post2 - 7, ".browse", 7))
 		post2 -= 7;
+	    nzFree(cw->referrer);
 	    cw->referrer = cloneString(currentReferrer);
 	    cw->referrer[post2 - currentReferrer] = 0;
 	    referrer = cw->referrer;
@@ -528,13 +566,13 @@ httpConnect(const char *from, const char *url)
 
     curlret = curl_easy_setopt(curl_handle, CURLOPT_REFERER, referrer);
     if(curlret != CURLE_OK)
-	goto curl_fail;;
+	goto curl_fail;
     curlret = curl_easy_setopt(curl_handle, CURLOPT_HTTPHEADER, custom_headers);
     if(curlret != CURLE_OK)
-	goto curl_fail;;
+	goto curl_fail;
     curlret = curl_easy_setopt(curl_handle, CURLOPT_URL, urlcopy);
     if(curlret != CURLE_OK)
-	goto curl_fail;;
+	goto curl_fail;
 
     /* If we have a username and password, then tell libcurl about it.
      * libcurl won't send it to the server unless server gave a 401 response.
@@ -545,15 +583,16 @@ httpConnect(const char *from, const char *url)
 	creds_len = strlen(creds_buf);
 	creds_buf[creds_len] = ':';
 	strcpy(creds_buf + creds_len + 1, pass);
-	creds_ptr = creds_buf;
-    } else if(getUserPass(urlcopy, creds_buf, false))
-	creds_ptr = creds_buf;
-    else
-	creds_ptr = NULL;
+    } else
+	getUserPass(urlcopy, creds_buf, false);
 
-    curlret = curl_easy_setopt(curl_handle, CURLOPT_USERPWD, creds_ptr);
+/*
+ * If the URL didn't have user and password, and getUserPass failed,
+ * then creds_buf == "".
+ */
+    curlret = curl_easy_setopt(curl_handle, CURLOPT_USERPWD, creds_buf);
     if(curlret != CURLE_OK)
-	goto curl_fail;;
+	goto curl_fail;
 
 /* We are ready to make a transfer.  Here is where it gets complicated.
  * At the top of the loop, we perform the HTTP request.  It may fail entirely
@@ -568,82 +607,84 @@ httpConnect(const char *from, const char *url)
  * then add it to the list of authentication records.  */
 
     still_fetching = true;
+    ssl_version = CURL_SSLVERSION_DEFAULT;
     serverData = initString(&serverDataLen);
 
     while(still_fetching == true) {
 	char *redir = NULL;
+	curl_easy_setopt(curl_handle, CURLOPT_SSLVERSION, ssl_version);
 	init_header_parser();
 	curlret = curl_easy_perform(curl_handle);
 	if(serverDataLen >= CHUNKSIZE)
 	    nl();		/* We printed dots, so we terminate them with newline */
+	if(curlret == CURLE_SSL_CONNECT_ERROR) {
+/* all this would be unnecessary if curl sent the proper hello message */
+/* try the next version */
+	    if(ssl_version == CURL_SSLVERSION_DEFAULT) {
+		ssl_version = CURL_SSLVERSION_SSLv3;
+		debugPrint(3, "stepping back to sslv3");
+		continue;
+	    }
+	    if(ssl_version == CURL_SSLVERSION_SSLv3) {
+		ssl_version = CURL_SSLVERSION_TLSv1;
+		debugPrint(3, "stepping back to tlsv1");
+		continue;
+	    }
+/* probably shouldn't step down to SSLv2; it is considered to be insecure */
+	}
 	if(curlret != CURLE_OK)
-	    goto curl_fail;;
+	    goto curl_fail;
 	curl_easy_getinfo(curl_handle, CURLINFO_RESPONSE_CODE, &hcode);
 	if(curlret != CURLE_OK)
-	    goto curl_fail;;
+	    goto curl_fail;
 
-	redir = get_redirect_location();
-	if(redir)
-	    redir = resolveURL(urlcopy, redir);
 	debugPrint(3, "http code %d %s", hcode, herror);
 
 	if(hcode >= 301 && hcode <= 303 && allowRedirection) {
+	    redir = get_redirect_location();
+	    if(redir)
+		redir = resolveURL(urlcopy, redir);
 	    still_fetching = false;
 	    if(redir == NULL) {
 		/* Redirected, but we don't know where to go. */
 		i_printf(MSG_RedirectNoURL, hcode);
 		transfer_status = true;
-	    } else {
-		creds_buf[0] = '\0';
-		temp_url = urlcopy;
+	    } else if(redirect_count >= 10) {
+		i_puts(MSG_RedirectMany);
+		transfer_status = true;
+		nzFree(redir);
+	    } else {		/* redirection looks good. */
+		strcpy(creds_buf, ":");	/* Flush stale data. */
+		nzFree(urlcopy);
 		urlcopy = redir;
 		unpercentURL(urlcopy);
 
-		if(redirect_count >= 10) {
-		    i_puts(MSG_RedirectMany);
-		    transfer_status = true;
-		} else {	/* redirection looks good. */
 /* Convert POST request to GET request after redirection. */
-		    curl_easy_setopt(curl_handle, CURLOPT_HTTPGET, 1);
+		curl_easy_setopt(curl_handle, CURLOPT_HTTPGET, 1);
 
-		    if(getUserPass(urlcopy, creds_buf, false))
-			creds_ptr = creds_buf;
-		    else
-			creds_ptr = NULL;
+		getUserPass(urlcopy, creds_buf, false);
 
-		    curlret =
-		       curl_easy_setopt(curl_handle, CURLOPT_USERPWD,
-		       creds_ptr);
-		    if(curlret != CURLE_OK) {
-			nzFree(temp_url);
-			goto curl_fail;
-		    }
+		curlret =
+		   curl_easy_setopt(curl_handle, CURLOPT_USERPWD, creds_buf);
+		if(curlret != CURLE_OK)
+		    goto curl_fail;
 
-		    if(sendReferrer) {
-			curlret = curl_easy_setopt(curl_handle, CURLOPT_REFERER,
-			   urlcopy);
-			if(curlret != CURLE_OK) {
-			    nzFree(temp_url);
-			    goto curl_fail;
-			}
-			nzFree(cw->referrer);
-			cw->referrer = cloneString(urlcopy);
-		    }
-		    nzFree(temp_url);
+		curlret = curl_easy_setopt(curl_handle, CURLOPT_URL, urlcopy);
+		if(curlret != CURLE_OK)
+		    goto curl_fail;
 
-		    curlret =
-		       curl_easy_setopt(curl_handle, CURLOPT_URL, urlcopy);
-		    if(curlret != CURLE_OK)
-			goto curl_fail;
+		nzFree(serverData);
+		serverData = EMPTYSTRING;
+		serverDataLen = 0;
+		redirect_count += 1;
+		still_fetching = true;
+		name_changed = true;
+		debugPrint(2, "redirect %s", urlcopy);
 
-		    nzFree(serverData);
-		    serverData = EMPTYSTRING;
-		    serverDataLen = 0;
-		    redirect_count += 1;
-		    still_fetching = true;
-		    name_changed = true;
-		    debugPrint(2, "redirect %s", urlcopy);
-		}
+/* after redirection, go back to default ssl version. */
+/* It might be a completely different server. */
+/* Some day we might want to cache which domains require which ssl versions */
+		ssl_version = CURL_SSLVERSION_DEFAULT;
 	    }
 	}
 
@@ -670,6 +711,8 @@ httpConnect(const char *from, const char *url)
     }
 
   curl_fail:
+    if(custom_headers)
+	curl_slist_free_all(custom_headers);
     if(curlret != CURLE_OK)
 	curl_setError(curlret, urlcopy);
 
@@ -677,6 +720,7 @@ httpConnect(const char *from, const char *url)
 	nzFree(serverData);
 	serverData = NULL;
 	serverDataLen = 0;
+	nzFree(urlcopy);	/* Free it on transfer failure. */
     } else {
 	if(hcode != 200)
 	    i_printf(MSG_HTTPError, hcode, message_for_response_code(hcode));
@@ -686,6 +730,7 @@ httpConnect(const char *from, const char *url)
 	    nzFree(urlcopy);	/* Don't need it anymore. */
     }
 
+    nzFree(postb);
     return transfer_status;
 }				/* httpConnect */
 
@@ -787,8 +832,8 @@ ftpls(char *line)
 
 /* parse_directory_listing: convert an FTP-style listing to html. */
 /* Repeatedly calls ftpls to parse each line of the data. */
-void
-parse_directory_listing()
+static void
+parse_directory_listing(void)
 {
     char *incomingData = serverData;
     int incomingLen = serverDataLen;
@@ -822,6 +867,10 @@ curl_setError(CURLcode curlret, const char *url)
     protocol = getProtURL(url);
     host = getHostURL(url);
 
+/* this should never happen */
+    if(!host)
+	host = EMPTYSTRING;
+
     switch (curlret) {
 
     case CURLE_UNSUPPORTED_PROTOCOL:
@@ -884,6 +933,9 @@ curl_setError(CURLcode curlret, const char *url)
 	setError(MSG_FTPTransfer);
 	break;
 
+    case CURLE_SSL_CONNECT_ERROR:
+	setError(MSG_SSLConnectError, errorText);
+	break;
     default:
 	setError(MSG_CurlCatchAll, curl_easy_strerror(curlret));
 	break;
@@ -1012,6 +1064,7 @@ allIPs(void)
 			ip = tcp_name_ip(dom);
 		    if(ip != NULL_IP) {
 /* could be a repeat */
+			debugPrint(3, "becomes %s", tcp_ip_dots(ip));
 			for(k = 0; k < iptotal; ++k)
 			    if(iplist[k] == ip)
 				break;
@@ -1053,8 +1106,8 @@ void
 my_curl_init(void)
 {
     const unsigned int major = 7;
-    const unsigned int minor = 15;
-    const unsigned int patch = 5;
+    const unsigned int minor = 17;
+    const unsigned int patch = 0;
     const unsigned int least_acceptable_version =
        (major << 16) | (minor << 8) | patch;
     curl_version_info_data *version_data = NULL;
@@ -1078,6 +1131,7 @@ my_curl_init(void)
     curl_easy_setopt(curl_handle, CURLOPT_PROGRESSFUNCTION, curl_progress);
     curl_easy_setopt(curl_handle, CURLOPT_CONNECTTIMEOUT, webTimeout);
     curl_easy_setopt(curl_handle, CURLOPT_USERAGENT, currentAgent);
+    curl_easy_setopt(curl_handle, CURLOPT_SSLVERSION, CURL_SSLVERSION_DEFAULT);
 
 /*
  * This next setting tells libcurl to use any available authentication type.
@@ -1085,7 +1139,12 @@ my_curl_init(void)
 */
     curl_easy_setopt(curl_handle, CURLOPT_HTTPAUTH, CURLAUTH_ANY);
 
-/* The next three setopt calls could allocate or perform file I/O. */
+/* The next few setopt calls could allocate or perform file I/O. */
+    errorText[0] = '\0';
+    curl_init_status =
+       curl_easy_setopt(curl_handle, CURLOPT_ERRORBUFFER, errorText);
+    if(curl_init_status != CURLE_OK)
+	goto libcurl_init_fail;
     curl_init_status = curl_easy_setopt(curl_handle, CURLOPT_CAINFO, sslCerts);
     if(curl_init_status != CURLE_OK)
 	goto libcurl_init_fail;
@@ -1097,7 +1156,8 @@ my_curl_init(void)
        curl_easy_setopt(curl_handle, CURLOPT_COOKIEJAR, cookieFile);
     if(curl_init_status != CURLE_OK)
 	goto libcurl_init_fail;
-    curl_easy_setopt(curl_handle, CURLOPT_PROXY, "");	/* No proxies for now. */
+    if(proxy_host)
+	curl_easy_setopt(curl_handle, CURLOPT_PROXY, proxy_host);
     atexit(my_curl_cleanup);
 
   libcurl_init_fail:
