@@ -38,19 +38,19 @@ my_ErrorReporter(JSContext * cx, const char *message, JSErrorReport * report)
     char *prefix, *tmp;
 
     if(debugLevel < 2)
-	return;
+	goto done;
     if(ismc)
-	return;
+	goto done;
 
     if(!report) {
 	fprintf(gErrFile, "%s\n", message);
-	return;
+	goto done;
     }
 
 /* Conditionally ignore reported warnings. */
     if(JSREPORT_IS_WARNING(report->flags)) {
 	if(browseLocal)
-	    return;
+	    goto done;
     }
 
     prefix = NULL;
@@ -76,6 +76,9 @@ my_ErrorReporter(JSContext * cx, const char *message, JSErrorReport * report)
 
     if(prefix)
 	JS_free(cx, prefix);
+
+  done:
+    report->flags = 0;
 }				/* my_ErrorReporter */
 
 
@@ -697,7 +700,7 @@ case 'span': return document.spans; \n\
 case 'head': return document.heads; \n\
 case 'meta': return document.metas; \n\
 case 'body': return document.bodies; \n\
-default: alert('all.tags default ' + s); return new Array(); }} \n\
+default: /* alert('all.tags default ' + s); */ return new Array(); }} \n\
 \n\
 document.getElementById = function(s) { \n\
 return document.idMaster[s]; } \n\
@@ -709,7 +712,7 @@ document.createElement = function(s) { \n\
 switch(s.toLowerCase()) { \n\
 case 'link': return new Link();\n\
 case 'image': case 'img': return new Image();\n\
-default: alert('createElement default ' + s); return new Object(); }} \n\
+default: /* alert('createElement default ' + s); */ return new Object(); }} \n\
 \n\
 URL.prototype.indexOf = function(s) { \n\
 return this.toString().indexOf(s); }\n\
@@ -788,6 +791,7 @@ createJavaContext(void)
     establish_property_bool(jwin, "toolbar", true, false);
     establish_property_bool(jwin, "resizable", true, false);
     establish_property_bool(jwin, "directories", false, false);
+    establish_property_string(jwin, "name", "unspecifiedFrame", false);
 
 /* Other classes that we'll need. */
     for(i = 0; domClasses[i].class; ++i) {
@@ -1082,13 +1086,19 @@ Yeah, it makes my head spin too.
 /* self-referencing - hope this is ok */
 		establish_property_object(v, "options", v);
 		establish_property_number(v, "selectedIndex", -1, false);
+// not the normal pathway; we have to create our own element methods here.
+		JS_DefineFunction(jcx, v, "focus", nullFunction, 0, PROP_FIXED);
+		JS_DefineFunction(jcx, v, "blur", nullFunction, 0, PROP_FIXED);
 	    }
 	} else {
 	    v = JS_NewObject(jcx, cp, NULL, owner);
 	}
 	vv = OBJECT_TO_JSVAL(v);
 
-	if(symname && !dupname) {
+/* if no name, then use id as name */
+	if(!symname && idname) {
+	    JS_DefineProperty(jcx, owner, idname, vv, NULL, NULL, attr);
+	} else if(symname && !dupname) {
 	    JS_DefineProperty(jcx, owner, symname, vv, NULL, NULL, attr);
 	    if(stringEqual(symname, "action"))
 		establish_property_bool(v, "actioncrash", true, true);
@@ -1112,6 +1122,8 @@ Yeah, it makes my head spin too.
 	    JS_DefineElement(jcx, alist, length, vv, NULL, NULL, attr);
 	    if(symname && !dupname)
 		establish_property_object(alist, symname, v);
+	    if(idname && (!symname || !stringEqual(symname, idname)))
+		establish_property_object(alist, idname, v);
 	}			/* list indicated */
     }
 
@@ -1134,6 +1146,8 @@ Yeah, it makes my head spin too.
 	JS_GetProperty(jcx, jdoc, "idMaster", &listv);
 	master = JSVAL_TO_OBJECT(listv);
 	establish_property_object(master, idname, v);
+    } else {
+	establish_property_string(v, "id", EMPTYSTRING, true);
     }
 
     if(href && href_url) {
