@@ -81,7 +81,7 @@ isWinLoc(void)
 	return true;
     }
 /* This call frees t, or takes it over, so you should not free it here. */
-    gotoLocation(uo_href, 0, true);
+    gotoLocation(uo_href, (allowRedirection ? 0 : 99), false);
     uo_href = 0;
     return false;
 }				/* isWinLoc */
@@ -101,7 +101,7 @@ loc_reload(JSContext * cx, JSObject * obj, uintN argc, jsval * argv,
 {
     const char *s = cw->firstURL;
     if(s && isURL(s))
-	gotoLocation(cloneString(s), 0, true);
+	gotoLocation(cloneString(s), (allowRedirection ? 0 : 99), true);
     else
 	JS_ReportError(jcx, "location.reload() cannot find a url to refresh");
     return JS_FALSE;
@@ -121,7 +121,7 @@ loc_replace(JSContext * cx, JSObject * obj, uintN argc, jsval * argv,
 	t = resolveURL(cw->fileName, ss);
 	nzFree(ss);
 /* This call frees t, or takes it over, so you should not free it here. */
-	gotoLocation(t, 0, true);
+	gotoLocation(t, (allowRedirection ? 0 : 99), true);
 	return JS_FALSE;
     }
     JS_ReportError(jcx,
@@ -281,7 +281,7 @@ setter_loc(JSContext * cx, JSObject * obj, jsval id, jsval * vp)
 	t = resolveURL(cw->fileName, ss);
 	nzFree(ss);
 /* This call frees t, or takes it over, so you should not free it here. */
-	gotoLocation(t, 0, true);
+	gotoLocation(t, (allowRedirection ? 0 : 99), false);
     }
 /* Return false to stop javascript. */
 /* After all, we're trying to move to a new web page. */
@@ -896,24 +896,14 @@ Compile and call event handlers.
 bool
 handlerGo(void *obj, const char *name)
 {
-    jsval v, rval;
+    jsval rval;
     bool rc;
-    JSObject *fo;		/* function object */
-    JSFunction *f;
     JSBool found;
     JS_HasProperty(jcx, obj, name, &found);
     if(!found)
 	return false;
-    JS_GetProperty(jcx, obj, name, &v);
-    if(!JSVAL_IS_OBJECT(v))
-	return false;
-    fo = JSVAL_TO_OBJECT(v);
-    if(!JS_ObjectIsFunction(jcx, fo))
-	return false;
-    f = JS_ValueToFunction(jcx, v);
-    JS_CallFunction(jcx, obj, f, 0, emptyArgs, &rval);
-    rc = true;
-    if(JSVAL_IS_BOOLEAN(rval))
+    rc = JS_CallFunctionName(jcx, obj, name, 0, emptyArgs, &rval);
+    if(rc && JSVAL_IS_BOOLEAN(rval))
 	rc = JSVAL_TO_BOOLEAN(rval);
     JS_GC(jcx);
     return rc;
