@@ -13,7 +13,7 @@
 /* See eb.h for descriptive comments. */
 
 
-const char *version = "3.4.1";
+const char *version = "3.4.5";
 char *userAgents[10], *currentAgent, *currentReferrer;
 const char eol[] = "\r\n";
 char EMPTYSTRING[] = "";
@@ -32,11 +32,11 @@ static struct DBTABLE dbtables[MAXDBT];
 char *dbarea, *dblogin, *dbpw;	/* to log into the database */
 bool fetchBlobColumns;
 char *proxy_host;
-int proxy_port;
 bool caseInsensitive, searchStringsAll;
 bool undoable;
 bool allowRedirection = true, allowJS = true, sendReferrer = false;
 bool binaryDetect = true;
+bool inputReadLine;
 bool showHiddenFiles, helpMessagesOn;
 uchar dirWrite, endMarks;
 int context = 1;
@@ -125,7 +125,7 @@ junkSubject(const char *s, char key)
     n = cfglen + l;
     new = allocMem(n);
     memcpy(new, cfgcopy, subjstart);
-    sprintf(new + subjstart, "%d`%s > x\n", exp, s);
+    sprintf(new + subjstart, "%ld`%s > x\n", exp, s);
     memcpy(new + subjstart + l, cfgcopy + subjstart, cfglen - subjstart);
     nzFree(cfgcopy);
     cfgcopy = new;
@@ -168,7 +168,7 @@ readConfigFile(void)
 	"adbook", "ipblack", "maildir", "agent",
 	"jar", "nojs", "spamcan",
 	"webtimer", "mailtimer", "certfile", "datasource", "proxy",
-	0
+	"linelength", 0
     };
 
     if(!fileTypeByName(configFile, false))
@@ -458,13 +458,17 @@ readConfigFile(void)
 
 	case 6:
 	    if(*v == '*')
-		act->inssl = true, ++v;
+		act->inssl = 1, ++v;
 	    act->inport = atoi(v);
 	    continue;
 
 	case 7:
+	    if(*v == '+')
+		act->outssl = 4, ++v;
+	    if(*v == '^')
+		act->outssl = 2, ++v;
 	    if(*v == '*')
-		act->outssl = true, ++v;
+		act->outssl = 1, ++v;
 	    act->outport = atoi(v);
 	    continue;
 
@@ -604,12 +608,12 @@ readConfigFile(void)
 
 	case 28:		/* proxy */
 	    proxy_host = v;
-	    proxy_port = 80;
-	    v = strchr(v, ':');
-	    if(v) {
-		*v++ = 0;
-		proxy_port = atoi(v);
-	    }
+	    continue;
+
+	case 29:		/* linelength */
+	    displayLength = atoi(v);
+	    if(displayLength < 80)
+		displayLength = 80;
 	    continue;
 
 	default:
@@ -1088,9 +1092,6 @@ main(int argc, char **argv)
 	i_printfExit(MSG_Usage);
     }				/* options */
 
-    if(debugLevel >= 4)
-	curl_easy_setopt(curl_handle, CURLOPT_VERBOSE, 1);
-
     if(tcp_init() < 0)
 	debugPrint(4, "tcp failure, could not identify this machine");
     else
