@@ -23,7 +23,7 @@ The array of messages is selected by the language.
 
 
 enum {
-    MSG_EOF,
+    MSG_EndFile,
     MSG_NoFile,
     MSG_None,
     MSG_SubGlobal,
@@ -675,4 +675,7 @@ enum {
     MSG_Fkeys,
     MSG_FetchBlobOff,
     MSG_FetchBlobOn,
+    MSG_SSLConnectError,
+    MSG_InputTTY,
+    MSG_InputReadLine,
 };
