@@ -10,11 +10,14 @@
 #include <time.h>
 
 #define MAXRECAT 100		/* max number of recipients or attachments */
+#define MAXMSLINE 1024		/* max mail server line */
 
-char serverLine[MAXTTYLINE];
-static char spareLine[MAXTTYLINE];
+char serverLine[MAXMSLINE];
+static char spareLine[MAXMSLINE];
 int mssock;			/* mail server socket */
 static bool doSignature;
+static bool ssl_on;
+static const char *mailhost;
 static char subjectLine[200];
 
 static struct ALIAS {
@@ -175,10 +178,42 @@ Print the lines if the debug level calls for it.
 *********************************************************************/
 
 bool
-serverPutLine(const char *buf, bool secure)
+mailPutLine(const char *buf, bool realdot)
 {
-    int n, len = strlen(buf);
+    int n, j, len = strlen(buf);
     char c;
+    static char last_nl = 0;
+    int dotcount = 0;
+    char *dotbuf = 0;
+    char *s;
+
+    s = strstr(buf, "\n.");
+    if(s && realdot && buf + len - s == 4)
+	s = 0;
+    if(last_nl && buf[0] == '.' || s) {
+// have to dot stuff
+// count dots first
+	if(last_nl && buf[0] == '.')
+	    ++dotcount;
+	for(n = 1; n < len; ++n)
+	    if(buf[n] == '.' && buf[n - 1] == '\n')
+		++dotcount;
+	dotbuf = allocMem(len + dotcount + 1);
+	j = 0;
+	if(last_nl && buf[0] == '.')
+	    dotbuf[j++] = '.';
+	for(n = 0; n < len; ++n) {
+	    c = buf[n];
+	    if(c == '.' && n && buf[n - 1] == '\n' &&
+	       (!realdot || len - n != 3))
+		dotbuf[j++] = '.';
+	    dotbuf[j++] = c;
+	}
+	dotbuf[j] = 0;
+	buf = dotbuf;
+	len = j;
+    }
+
     if(debugLevel >= 4) {
 	printf("> ");
 	for(n = 0; n < len; ++n) {
@@ -187,28 +222,44 @@ serverPutLine(const char *buf, bool secure)
 		putchar(c);
 	}
     }
-    if(secure)
+
+    if(len) {
+	if(buf[len - 1] == '\n')
+	    last_nl = 1;
+	else
+	    last_nl = 0;
+    }
+
+    if(ssl_on)
 	n = ssl_write(buf, len);
     else
 	n = tcp_write(mssock, buf, len);
+
+    nzFree(dotbuf);
+
     if(n < len) {
 	setError(MSG_MailWrite);
 	return false;
     }
     return true;
-}				/* serverPutLine */
+}				/* mailPutLine */
 
 bool
-serverGetLine(bool secure)
+mailGetLine(void)
 {
     int n, len, slen;
     char c;
     char *s;
     slen = strlen(spareLine);
     strcpy(serverLine, spareLine);
+  top:
     s = strchr(serverLine, '\n');
-    if(!s) {
-	if(secure)
+    while(!s) {
+	if(slen + 1 == sizeof (serverLine)) {
+	    setError(MSG_MailResponseLong);
+	    return false;
+	}
+	if(ssl_on)
 	    len = ssl_read(serverLine + slen, sizeof (serverLine) - 1 - slen);
 	else
 	    len =
@@ -220,11 +271,7 @@ serverGetLine(bool secure)
 	}
 	slen += len;
 	serverLine[slen] = 0;
-    }
-    s = strchr(serverLine, '\n');
-    if(!s) {
-	setError(MSG_MailResponseLong);
-	return false;
+	s = strchr(serverLine, '\n');
     }
     strcpy(spareLine, s + 1);
     *s = 0;
@@ -232,39 +279,49 @@ serverGetLine(bool secure)
 	*--s = 0;
     debugPrint(4, "< %s", serverLine);
     return true;
-}				/* serverGetLine */
+}				/* mailGetLine */
 
 static bool
-serverPutGet(const char *line, bool secure)
+mailPutGet(const char *line)
 {
-    if(!serverPutLine(line, secure))
+    if(!mailPutLine(line, false))
 	return false;
-    if(!serverGetLine(secure))
+    if(!mailGetLine())
 	return false;
     return true;
-}				/* serverPutGet */
+}				/* mailPutGet */
 
 void
-serverClose(bool secure)
+mailPutGetError(const char *line)
 {
-    serverPutLine("quit\r\n", secure);
+    if(!mailPutGet(line))
+	showErrorAbort();
+}				/* mailPutGetError */
+
+void
+mailClose(void)
+{
+    mailPutLine("quit\r\n", false);
     endhostent();
-    sleep(2);
-    if(secure)
+/* the other side has to have time to process the quit command, before we simply hang up. */
+    usleep(400000);
+    if(ssl_on)
 	ssl_done();
     close(mssock);
-}				/* serverClose */
+}				/* mailClose */
 
 /* Connect to the mail server */
 bool
-mailConnect(const char *host, int port, bool secure)
+mailConnect(const char *host, int port, int secure)
 {
+    ssl_on = false;
     IP32bit ip = tcp_name_ip(host);
     if(ip == NULL_IP) {
 	setError((intFlag ? MSG_Interrupted : MSG_MailLocate), host);
 	return false;
     }
     debugPrint(4, "%s -> %s", host, tcp_ip_dots(ip));
+    mailhost = host;
     mssock = tcp_connect(ip, port, mailTimeout);
     if(mssock < 0) {
 	setError(intFlag ? MSG_Interrupted : MSG_MailConnect);
@@ -272,7 +329,7 @@ mailConnect(const char *host, int port, bool secure)
     }
     debugPrint(4, "connected to port %d", port);
     spareLine[0] = 0;
-    if(secure) {
+    if(secure & 1) {
 	int n = ssl_newbind(mssock);
 	if(n < 0) {
 	    if(n == -999)
@@ -281,7 +338,8 @@ mailConnect(const char *host, int port, bool secure)
 		setError(MSG_WebConnectSecure, host, n);
 	    return false;
 	}
-	debugPrint(4, "secure connection established");
+	debugPrint(3, "secure connection established");
+	ssl_on = true;
     }
     return true;
 }				/* mailConnect */
@@ -802,6 +860,7 @@ sendMail(int account, const char **recipients, const char *body,
     int nat, cx, i, j;
     char *out = 0;
     bool mustmime = false;
+    bool firstgreet = true;
     bool firstrec;
     const char *ct, *ce;
     char *encoded = 0;
@@ -920,10 +979,10 @@ sendMail(int account, const char **recipients, const char *body,
 	nzFree(encoded);
 	return false;
     }
-    if(!serverGetLine(ao->outssl))
+    if(!mailGetLine())
 	goto mailfail;
     while(memEqualCI(serverLine, "220-", 4)) {
-	if(!serverGetLine(ao->outssl))
+	if(!mailGetLine())
 	    goto mailfail;
     }
     if(!memEqualCI(serverLine, "220 ", 4)) {
@@ -931,11 +990,12 @@ sendMail(int account, const char **recipients, const char *body,
 	goto mailfail;
     }
 
+  sayhello:
     sprintf(serverLine, "%s %s%s", (a->outssl ? "ehlo" : "Helo"), smlogin, eol);
-    if(!serverPutLine(serverLine, ao->outssl))
+    if(!mailPutLine(serverLine, false))
 	goto mailfail;
   get250:
-    if(!serverGetLine(ao->outssl))
+    if(!mailGetLine())
 	goto mailfail;
     if(!memEqualCI(serverLine, "250", 3) ||
        serverLine[3] != ' ' && serverLine[3] != '-') {
@@ -947,8 +1007,34 @@ sendMail(int account, const char **recipients, const char *body,
 
     if(ao->outssl) {
 	char *b;
+	int nb;
+
+	if(a->outssl == 2 && firstgreet) {
+/* haven't yet switched over to ssl */
+	    if(!mailPutGet("starttls\r\n"))
+		goto mailfail;
+	    if(!memEqualCI(serverLine, "220 ", 4)) {
+		setError(MSG_MailBadPrompt, serverLine);
+		goto mailfail;
+	    }
+	    nb = ssl_newbind(mssock);
+	    if(nb < 0) {
+		if(nb == -999)
+		    setError(MSG_NoCertify, mailhost);
+		else
+		    setError(MSG_WebConnectSecure, mailhost, nb);
+		goto mailfail;
+	    }
+	    debugPrint(3, "secure connection established");
+	    ssl_on = true;
+
+/* We have to send EHLO after starttls, per RFC 2487 */
+	    firstgreet = false;
+	    goto sayhello;
+	}
+
 /* login authentication is the only thing I support right now. */
-	if(!serverPutGet("auth login\r\n", true))
+	if(!mailPutGet("auth login\r\n"))
 	    goto mailfail;
 	if(!memEqualCI(serverLine, "334 ", 4)) {
 	    setError(MSG_AuthLoginOnly);
@@ -957,7 +1043,7 @@ sendMail(int account, const char **recipients, const char *body,
 	b = base64Encode(login, strlen(login), false);
 	sprintf(serverLine, "%s%s", b, eol);
 	nzFree(b);
-	if(!serverPutGet(serverLine, true))
+	if(!mailPutGet(serverLine))
 	    goto mailfail;
 	if(!memEqualCI(serverLine, "334 ", 4)) {
 	    setError(MSG_AuthLoginOnly);
@@ -966,7 +1052,7 @@ sendMail(int account, const char **recipients, const char *body,
 	b = base64Encode(pass, strlen(pass), false);
 	sprintf(serverLine, "%s%s", b, eol);
 	nzFree(b);
-	if(!serverPutGet(serverLine, true))
+	if(!mailPutGet(serverLine))
 	    goto mailfail;
 	if(!memEqualCI(serverLine, "235 ", 4)) {
 	    setError(MSG_SmtpNotComplete, serverLine);
@@ -975,7 +1061,7 @@ sendMail(int account, const char **recipients, const char *body,
     }
 
     sprintf(serverLine, "mail from: <%s>%s", reply, eol);
-    if(!serverPutGet(serverLine, ao->outssl))
+    if(!mailPutGet(serverLine))
 	goto mailfail;
     if(!memEqualCI(serverLine, "250 ", 4)) {
 	setError(MSG_MailReject, reply);
@@ -984,7 +1070,7 @@ sendMail(int account, const char **recipients, const char *body,
 
     for(j = 0; s = recipients[j]; ++j) {
 	sprintf(serverLine, "rcpt to: <%s>%s", s, eol);
-	if(!serverPutGet(serverLine, ao->outssl))
+	if(!mailPutGet(serverLine))
 	    goto mailfail;
 	if(!memEqualCI(serverLine, "250 ", 4)) {
 	    setError(MSG_MailReject, s);
@@ -992,7 +1078,7 @@ sendMail(int account, const char **recipients, const char *body,
 	}
     }
 
-    if(!serverPutGet("data\r\n", ao->outssl))
+    if(!mailPutGet("data\r\n"))
 	goto mailfail;
     if(!memEqualCI(serverLine, "354 ", 4)) {
 	setError(MSG_MailNotReady, serverLine);
@@ -1045,9 +1131,9 @@ sendMail(int account, const char **recipients, const char *body,
     if(fromiso != from)
 	nzFree(fromiso);
     if(refline) {
-	s = refline + strlen(refline);
-	if(s > refline && s[-1] == '\n')
-	    --s;
+	s = strchr(refline, '\n');
+	if(!s)			/* should never happen */
+	    s = refline + strlen(refline);
 	stringAndBytes(&out, &j, refline, s - refline);
 	stringAndString(&out, &j, eol);
     }
@@ -1114,12 +1200,12 @@ this format, some or all of this message may not be legible.\r\n\r\n--");
     /* mime format */
     /* A dot alone ends the transmission */
     stringAndString(&out, &j, ".\r\n");
-    if(!serverPutLine(out, ao->outssl))
+    if(!mailPutLine(out, true))
 	goto mailfail;
     nzFree(out);
     out = 0;
 
-    if(!serverGetLine(ao->outssl))
+    if(!mailGetLine())
 	goto mailfail;
     if(!memEqualCI(serverLine, "250 ", 4) &&
 /* do these next two lines make any sense? */
@@ -1129,7 +1215,7 @@ this format, some or all of this message may not be legible.\r\n\r\n--");
 	goto mailfail;
     }
 
-    serverClose(ao->outssl);
+    mailClose();
     return true;
 
   mailfail:
